@@ -93,6 +93,12 @@
 #define ABS_MT_SLOT 0x2f
 #endif
 
+/* 4.15 */
+
+#ifndef BTN_STYLUS3
+#define BTN_STYLUS3 0x149
+#endif
+
 /******************************************************************************
  * Forward Declarations
  *****************************************************************************/
@@ -169,6 +175,7 @@ struct _WacomModel
 #define ABSOLUTE_FLAG		0x00000100
 #define BAUD_19200_FLAG		0x00000400
 #define BUTTONS_ONLY_FLAG	0x00000800
+#define SCROLLMODE_FLAG		0x00001000
 
 #define IsCursor(priv) (DEVICE_ID((priv)->flags) == CURSOR_ID)
 #define IsStylus(priv) (DEVICE_ID((priv)->flags) == STYLUS_ID)
@@ -182,8 +189,8 @@ struct _WacomModel
 
 #define FILTER_PRESSURE_RES	65536	/* maximum points in pressure curve */
 /* Tested result for setting the pressure threshold to a reasonable value */
-#define THRESHOLD_TOLERANCE (FILTER_PRESSURE_RES / 125)
-#define DEFAULT_THRESHOLD (FILTER_PRESSURE_RES / 75)
+#define THRESHOLD_TOLERANCE (0.008f)
+#define DEFAULT_THRESHOLD (0.013f)
 
 #define WCM_MAX_BUTTONS		32	/* maximum number of tablet buttons */
 #define WCM_MAX_X11BUTTON	127	/* maximum button number X11 can handle */
@@ -282,9 +289,11 @@ struct _WacomDeviceRec
 	WacomCommonPtr common;  /* common info pointer */
 
 	/* state fields in device coordinates */
+	struct _WacomDeviceState wcmPanscrollState; /* panscroll state tracking */
 	struct _WacomDeviceState oldState; /* previous state information */
 	int oldCursorHwProx;	/* previous cursor hardware proximity */
 
+	int maxCurve;		/* maximum pressure curve value */
 	int *pPressCurve;       /* pressure curve */
 	int nPressCtrl[4];      /* control points for curve */
 	int minPressure;	/* the minimum pressure a pen may hold */
@@ -460,6 +469,7 @@ struct _WacomCommonRec
 	int wcmRawSample;	     /* Number of raw data used to filter an event */
 	int wcmPressureRecalibration; /* Determine if pressure recalibration of
 					 worn pens should be performed */
+	int wcmPanscrollThreshold;	/* distance pen must move to send a panscroll event */
 
 	int bufpos;                        /* position with buffer */
 	unsigned char buffer[BUFFER_SIZE]; /* data read from device */
